`timescale 1ps / 1ps
//*****************************************************************************
// File Name            : stepmotor_ctrl.v
//-----------------------------------------------------------------------------
// Function             : stepmotor controller
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo 
//-----------------------------------------------------------------------------
// History
// -.-- 2012/9/18@a
//*****************************************************************************

module stepmotor_ctrl (
  clk,
  rstb,
  sw_0,
  sw_1,
  sw_2,
  sw_3,
  pwm_1,
  pwm_1b,
  pwm_2,
  pwm_2b,
  led,
  led_sp
);

  input clk;
  input rstb;
  input sw_0;
  input sw_1;
  input sw_2;
  input sw_3;
  output pwm_1;
  output pwm_1b;
  output pwm_2;
  output pwm_2b;
  output led;
  output [3:0]led_sp;
   
  reg pwm_1;
  reg pwm_1b;
  reg pwm_2;
  reg pwm_2b;
  reg [15:0] cnt_ms_base;
  reg [9:0] cnt_pwm_base;
  reg [9:0] cnt_pwm_base_max;
  reg [1:0] cnt_pwm;

  reg       sw_2_delay;
  reg       sw_3_delay;
  reg       sw_2_sig;
  reg       sw_3_sig;
 
  wire     ms_sig;
  wire    pwm_en;

parameter p_1ms_cnt = 39999 ;//CLK=40MHẑƂ1ms̃JEgl
   
//1ms̃JE^   
always @ (posedge clk or negedge rstb )
if (rstb==1'b0) 
  cnt_ms_base <= 16'd0;
else
  if (ms_sig == 1'd1)
    cnt_ms_base <= 16'd0;
  else
    cnt_ms_base <= cnt_ms_base + 16'd1;

assign  ms_sig = (cnt_ms_base == 16'd39999)?1'b1:1'b0; //1ms1clk1ɂȂ
   
//[^[JE^
always @ (posedge clk or negedge rstb )
if (rstb==1'b0) 
  cnt_pwm_base <= 10'd0;
else
  if (ms_sig == 1'b1)
    //cnt_pwm_base_max̒lŎύX
    if (cnt_pwm_base >= cnt_pwm_base_max)
      cnt_pwm_base <= 2'd0;
    else
      cnt_pwm_base <= cnt_pwm_base + 10'd1;
  else  
   cnt_pwm_base <= cnt_pwm_base ;


//[^[MMXVCl[u
assign pwm_en =  ((cnt_pwm_base == cnt_pwm_base_max)&&(ms_sig == 1'b1))?1'b1:1'b0;
   
//[^[]JE^   
always @ (posedge clk or negedge rstb )
if (rstb==1'b0) 
  cnt_pwm <= 2'd0;
else
  if (pwm_en == 1'b1)
    if (cnt_pwm == 2'd3)
      cnt_pwm <= 2'd0;
    else
      cnt_pwm <= cnt_pwm + 2'd1;
  else  
    cnt_pwm <= cnt_pwm ;

//pXP   
always @ (posedge clk or negedge rstb )
if (rstb==1'b0) 
  pwm_1 <= 2'd1;
else
  if((sw_0==1'b0)||(sw_1==1'b0))
    case (cnt_pwm)
      2'd0:pwm_1 <= 2'd1;
      2'd1:pwm_1 <= 2'd1;
      2'd2:pwm_1 <= 2'd0;
      2'd3:pwm_1 <= 2'd0;
    endcase // case(cnt_pwm)
  else 
    pwm_1 <= 2'd1;
  
//pXPt   
always @ (posedge clk or negedge rstb )
if (rstb==1'b0) 
  pwm_1b <= 2'd0;
else
  if((sw_0==1'b0)||(sw_1==1'b0))
    case (cnt_pwm)
      2'd0:pwm_1b <= 2'd0;
      2'd1:pwm_1b <= 2'd0;
      2'd2:pwm_1b <= 2'd1;
      2'd3:pwm_1b <= 2'd1;
    endcase // case(cnt_pwm)
  else 
    pwm_1b <= 2'd0;

//pX2   
always @ (posedge clk or negedge rstb )
if (rstb==1'b0) 
  pwm_2 <= 2'd0;
else
  if(sw_0==1'b0)
    case (cnt_pwm)
      2'd0:pwm_2 <= 2'd0;
      2'd1:pwm_2 <= 2'd1;
      2'd2:pwm_2 <= 2'd1;
      2'd3:pwm_2 <= 2'd0;
    endcase // case(cnt_pwm)
  else if(sw_1==1'b0)
    case (cnt_pwm)
      2'd0:pwm_2 <= 2'd1;
      2'd1:pwm_2 <= 2'd0;
      2'd2:pwm_2 <= 2'd0;
      2'd3:pwm_2 <= 2'd1;
    endcase // case(cnt_pwm)
  else 
    pwm_2 <= 2'd0;
  
//pX2t   
always @ (posedge clk or negedge rstb )
if (rstb==1'b0) 
  pwm_2b <= 2'd1;
else
  if(sw_0==1'b0)
    case (cnt_pwm)
      2'd0:pwm_2b <= 2'd1;
      2'd1:pwm_2b <= 2'd0;
      2'd2:pwm_2b <= 2'd0;
      2'd3:pwm_2b <= 2'd1;
    endcase // case(cnt_pwm)
  else if(sw_1==1'b0)
    case (cnt_pwm)
      2'd0:pwm_2b <= 2'd0;
      2'd1:pwm_2b <= 2'd1;
      2'd2:pwm_2b <= 2'd1;
      2'd3:pwm_2b <= 2'd0;
    endcase // case(cnt_pwm)
  else 
    pwm_2b <= 2'd1;
      

assign led = ((sw_0==1'b0)||(sw_1==1'b0)||(sw_2==1'b0)||(sw_3==1'b0))?1'b0:1'b1;

//sw_21msxM
always @ (posedge clk or negedge rstb )
if (rstb==1'b0)
  sw_2_delay <= 1'b1;
else
  if (ms_sig==1'b1)
    sw_2_delay <= sw_2;
  else
    sw_2_delay <= sw_2_delay;

//sw_2̗茟o
always @ (posedge clk or negedge rstb )
if (rstb==1'b0)
  sw_2_sig <= 1'b0;
else   
  if ((ms_sig==1'b1)&&(sw_2==1'b0)&&(sw_2_delay==1'b1))
    sw_2_sig <= 1'b1;
  else
    sw_2_sig <= 1'b0;

//sw_31msxM
always @ (posedge clk or negedge rstb )
if (rstb==1'b0)
  sw_3_delay <= 1'b1;
else
  if (ms_sig==1'b1)
    sw_3_delay <= sw_3;
  else
    sw_3_delay <= sw_3_delay;   

//sw_3̗茟o
always @ (posedge clk or negedge rstb )
if (rstb==1'b0)
  sw_3_sig <= 1'b0;
else   
  if ((ms_sig==1'b1)&&(sw_3==1'b0)&&(sw_3_delay==1'b1))
    sw_3_sig <= 1'b1;
  else
    sw_3_sig <= 1'b0;

//[^[JE^̍őlݒ
always @ (posedge clk or negedge rstb )
if (rstb==1'b0) 
  cnt_pwm_base_max <= 10'd9;
else
  if (sw_2_sig==1'b1) //xAbv
    cnt_pwm_base_max <= cnt_pwm_base_max - 10'd1;
  else if (sw_3_sig==1'b1) //x_E
    cnt_pwm_base_max <= cnt_pwm_base_max + 10'd1;
  else
    cnt_pwm_base_max <= cnt_pwm_base_max ; 

assign led_sp =  ~cnt_pwm_base_max[3:0];
   

      
endmodule






